/*
LDC1000 Barometric Pressure Sensor Display
 
 
 Circuit:
 LDC1000 sensor attached to pins 7,8,14,15:

 CSB: pin 8
 MOSI: pin 15
 MISO: pin 14
 SCK: pin 7
  
 created 16 Oct July 2014
 modified 
 by Regina
 
 */

// the sensor communicates using SPI, so include the library:
#include <SPI.h>

const int RPMIN  = 0x3C;
const int RPMAX  = 0x13;

//Sensor's memory register addresses:
// LDC COMMANDS
const int   LDC1000_CMD_REVID 		=0x00;
const int   LDC1000_CMD_RPMAX 		=0x01;
const int   LDC1000_CMD_RPMIN 		=0x02;
const int   LDC1000_CMD_SENSORFREQ 	=0x03;
const int   LDC1000_CMD_LDCCONFIG 	=0x04;
const int   LDC1000_CMD_CLKCONFIG 	=0x05;
const int   LDC1000_CMD_THRESHILSB 	=0x06;
const int   LDC1000_CMD_THRESHIMSB 	=0x07;
const int   LDC1000_CMD_THRESLOLSB 	=0x08;
const int   LDC1000_CMD_THRESLOMSB 	=0x09;
const int   LDC1000_CMD_INTCONFIG 	=0x0A;
const int   LDC1000_CMD_PWRCONFIG 	=0x0B;
const int   LDC1000_CMD_STATUS	 	=0x20;
const int   LDC1000_CMD_PROXLSB 	=0x21;
const int   LDC1000_CMD_PROXMSB 	=0x22;
const int   LDC1000_CMD_FREQCTRLSB	=0x23;
const int   LDC1000_CMD_FREQCTRMID	=0x24;
const int   LDC1000_CMD_FREQCTRMSB	=0x25;

// LDC BITMASKS
const int   LDC1000_BIT_AMPLITUDE    =0x18;
const int   LDC1000_BIT_RESPTIME     =0x07;
const int   LDC1000_BIT_CLKSEL       =0x02;
const int   LDC1000_BIT_CLKPD        =0x01;
const int   LDC1000_BIT_INTMODE      =0x07;
const int   LDC1000_BIT_PWRMODE      =0x01;
const int   LDC1000_BIT_STATUSOSC    =0x80;
const int   LDC1000_BIT_STATUSDRDYB  =0x40;
const int   LDC1000_BIT_STATUSWAKEUP =0x20;
const int   LDC1000_BIT_STATUSCOMP   =0x10;

//Read and Write command
const byte RWMASK = 0x80;

// pins used for the connection with the sensor
// the other you need are controlled by the SPI library):
const int dataReadyPin = 9;  // P2.1
const int chipSelectPin = 8; // P2.7
const int LDCClockPin = RED_LED;

void setup() {
  Serial.begin(9600);

  pinMode(dataReadyPin, INPUT);
  pinMode(chipSelectPin, OUTPUT);
  digitalWrite(chipSelectPin, HIGH);
  digitalWrite(chipSelectPin, LOW);
  // start the SPI library:
  SPI.begin();
  
  // initialize the  data ready and chip select pins:
 
  DCLKInit(); //LDC1000frequency clock init, uing P1.0 as ACLK@32kHz

//Configure LDC1000 register
  writeRegister(LDC1000_CMD_PWRCONFIG,   0x00);
  delay(100);

  writeRegister(LDC1000_CMD_RPMAX,       RPMAX);
  writeRegister(LDC1000_CMD_RPMIN,       RPMIN);
  writeRegister(LDC1000_CMD_SENSORFREQ,  0x91);
  writeRegister(LDC1000_CMD_LDCCONFIG,   0x17);
  writeRegister(LDC1000_CMD_CLKCONFIG,   0x02);
  writeRegister(LDC1000_CMD_INTCONFIG,   0x02);
  
  writeRegister(LDC1000_CMD_THRESHILSB,  0x50);
  writeRegister(LDC1000_CMD_THRESHIMSB,  0x14);
  writeRegister(LDC1000_CMD_THRESLOLSB,  0xC0);
  writeRegister(LDC1000_CMD_THRESLOMSB,  0x12);
  
  writeRegister(LDC1000_CMD_PWRCONFIG,   0x01);
  // give the sensor time to set up:
  delay(100);
  
}

void loop() {
  
  int devID = 0;
  
  devID = readRegister(LDC1000_CMD_REVID,1);
  delay(2000);
  Serial.print("device ID is:");
  Serial.println(devID,HEX);
//  
//  uint16_t proximtyData = readRegister(LDC1000_CMD_PROXLSB,2);
//  Serial.print("proximry data is:");
//  Serial.println(proximtyData);
//  
//  uint16_t frequencyData = readRegister(LDC1000_CMD_FREQCTRLSB,2);
//  Serial.print("frequency data is:");
//  Serial.println(frequencyData);
 
  //writeRegister(LDC1000_CMD_RPMAX,0x11);
  uint16_t frequencyData = readRegister(LDC1000_CMD_RPMAX,2);
  delay(500);
  Serial.print("frequency data is:");
  Serial.println(frequencyData,HEX);
  
  frequencyData = readRegister(LDC1000_CMD_RPMAX+2,2);
  delay(500);
  Serial.print("frequency data is:");
  Serial.println(frequencyData,HEX);
  
  frequencyData = readRegister(LDC1000_CMD_RPMAX+4,2);
  delay(500);
  Serial.print("frequency data is:");
  Serial.println(frequencyData,HEX);
  
  frequencyData = readRegister(LDC1000_CMD_RPMAX+6,2);
  delay(500);
  Serial.print("frequency data is:");
  Serial.println(frequencyData,HEX);
}


void DCLKInit()
{
  uint8_t bit = digitalPinToBitMask(RED_LED); // get pin bit
  uint8_t port = digitalPinToPort(RED_LED);   // get pin port
 
  volatile uint8_t *sel;
  sel = portSelRegister(port);
  
  pinMode(RED_LED,OUTPUT);
  *sel |= bit;
}

//Read from or write to register from the LDC1000:
unsigned int readRegister(byte thisRegister, int bytesToRead ) {
  byte inByte = 0;           // incoming byte from the SPI
  unsigned int result = 0;   // result to return
  Serial.print(thisRegister, HEX);
  Serial.print("\t");
  
  // now combine the address and the command into one byte
  byte dataToSend = thisRegister | RWMASK;
  Serial.println(dataToSend, HEX);
  // take the chip select low to select the device:
  digitalWrite(chipSelectPin, LOW);
  // send the device the register you want to read:
  SPI.transfer(dataToSend);
  // send a value of 0 to read the first byte returned:
  result = SPI.transfer(0x00);
  // decrement the number of bytes left to read:
  bytesToRead--;
  // if you still have another byte to read:
  if (bytesToRead > 0) {
    // shift the first byte left, then get the second byte:
    result = result << 8;
    inByte = SPI.transfer(0x00);
    // combine the byte you just got with the previous one:
    result = result | inByte;
    // decrement the number of bytes left to read:
    bytesToRead--;
  }
  // take the chip select high to de-select:
  digitalWrite(chipSelectPin, HIGH);
  // return the result:
  return(result);
}


  //Sends a write command to LDC1000

void writeRegister(byte thisRegister, byte thisValue) {


  // now combine the register address and the command into one byte:
  byte dataToSend = thisRegister & ~RWMASK;
  
  Serial.print(thisRegister, HEX); 
  Serial.print("\t");
  Serial.println(dataToSend, HEX);
  // take the chip select low to select the device:
  digitalWrite(chipSelectPin, LOW);

  SPI.transfer(dataToSend); //Send register location
  SPI.transfer(thisValue);  //Send value to record into register

  // take the chip select high to de-select:
  digitalWrite(chipSelectPin, HIGH);
}
